﻿#include <QApplication>
#include <QMainWindow>
#include <QPushButton>
#include <QDebug>
#include <QHBoxLayout>
#include <QScrollArea>
#include <QStackedLayout>
#include <QScrollBar>

class EventLogger : public QObject
{
public:
    EventLogger(QObject* parent):QObject(parent){}
protected:
    bool eventFilter(QObject *obj, QEvent *event) override {
        switch (event->type()){
            case QEvent::Resize:
                qDebug() << "---------------------------------------------------------";
                qDebug() << "object: " << obj << " event: " << event->type();
                break;
        }
        return QObject::eventFilter(obj, event);
    }
};


class DeleteOnHide : public QObject
{
public:
    DeleteOnHide(QObject* parent):QObject(parent){}
protected:
    bool eventFilter(QObject *obj, QEvent *event) override {
        if (event->type() == QEvent::Hide) {
            obj->deleteLater();
        }
        return QObject::eventFilter(obj, event);
    }
};

void installLogger(QScrollArea* scroll){
    scroll->viewport()->installEventFilter(new EventLogger(scroll->viewport()));
    QObject::connect(scroll->horizontalScrollBar(), &QScrollBar::rangeChanged, [scroll](int min, int max){
        qDebug() << "object: " << scroll << " H. bar range changed: (" << min << ", " << max << ")";
    });
    QObject::connect(scroll->verticalScrollBar(), &QScrollBar::rangeChanged, [scroll](int min, int max){
        qDebug() << "object: " << scroll << " V. bar range changed: (" << min << ", " << max << ")";
    });
}

QWidget* makeMainWidget();

void makeMainWidgetAndShow(const QSize& size){
    auto widget = makeMainWidget();
    widget->installEventFilter(new DeleteOnHide(widget));
    // show before resize does not cause infinite loop
    widget->resize(size);
    widget->show();
}

QWidget* makeMainWidget(){
    auto widget = new QWidget();
    // QStackedLayout is needed as outer layout
    auto stackedLayout = new QStackedLayout;
    {
        auto scroll = new QScrollArea();
        installLogger(scroll);

        // widgetResizable is needed
        scroll->setWidgetResizable(true);
        {
            auto innerWidget = new QWidget();
            auto innerWidgetLayout = new QHBoxLayout();
            // sizeConstraint is needed to be SetFixedSize
            innerWidgetLayout->setSizeConstraint(QLayout::SetFixedSize);
            innerWidget->setLayout(innerWidgetLayout);
            {
                auto pushbutton = new QPushButton();
                // the content is needed to be larger than dialog's default size
                pushbutton->setMinimumSize(QSize(600,100));
                pushbutton->setText("Clone");
                QObject::connect(pushbutton, &QPushButton::clicked, [widget](){
                    makeMainWidgetAndShow(QSize(630, 125));
                });

                innerWidgetLayout->addWidget(pushbutton);
            }
            scroll->setWidget(innerWidget);
        }
        stackedLayout->addWidget(scroll);
    }
    widget->setLayout(stackedLayout);

    return widget;
}

int main(int argc, char **argv){
    QApplication app(argc, argv);
    auto widget = makeMainWidget();
    widget->show();
    app.exec();
    delete widget;
    return 0;
}
